/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <SDL/SDL_types.h>

#include "filer.h"
#include "types.h"

char *filer_error = "";

void filer_init(void) {
}

int filer_open(char *filename, int mode, FILER_FD *fd) {
	if (mode & FILER_WRITE)
		*fd = open(filename, O_CREAT|O_WRONLY, 0644);
	else
		*fd = open(filename, O_RDONLY);
	if (*fd == -1) {
		filer_error = strerror(errno);
		return -1;
	}
	return 0;
}

int filer_read(FILER_FD fd, void *buffer, Uint32 size) {
	Uint32 count = 0, tries = 3;
	int r;
	do {
		do {
			r = read(fd, buffer, size);
		} while (r == -1 && errno == EINTR && --tries);
		if (r == -1) {
			filer_error = strerror(errno);
			return -1;
		}
		size -= r;
		buffer += r;
		count += r;
	} while (r && size > 0);
	return count;
}

int filer_write(FILER_FD fd, void *buffer, Uint32 size) {
	Uint32 count = 0, tries = 3;
	int r;
	do {
		do {
			r = write(fd, buffer, size);
		} while (r == -1 && errno == EINTR && --tries);
		if (r == -1) {
			filer_error = strerror(errno);
			return -1;
		}
		size -= r;
		buffer += r;
		count += r;
	} while (r && size > 0);
	return count;
}

void filer_close(FILER_FD fd) {
	close(fd);
}

int filer_load_file(char *filename, void *buf, int size) {
	int count;
	FILER_FD fd;
	if (filer_open(filename, FILER_READ, &fd) == -1)
		return -1;
	count = filer_read(fd, buf, size);
	filer_close(fd);
	return count;
}

int filer_write_byte(FILER_FD fd, Uint8 octet) {
	return filer_write(fd, &octet, 1);
}

int filer_write_word16(FILER_FD fd, Uint16 word16) {
	if (filer_write_byte(fd, word16 >> 8) == -1)
		return -1;
	return filer_write_byte(fd, word16 & 0xff);
}

int filer_write_word32(FILER_FD fd, Uint32 word32) {
	if (filer_write_word16(fd, word32 >> 16) == -1)
		return -1;
	return filer_write_word16(fd, word32 & 0xffff);
}

int filer_read_byte(FILER_FD fd, Uint8 *dest) {
	return filer_read(fd, dest, 1);
}

int filer_read_word16(FILER_FD fd, Uint16 *dest) {
	Uint8 octet;
	int ret;
	if (filer_read(fd, &octet, 1) == -1)
		return -1;
	*dest = octet << 8;
	if ((ret = filer_read(fd, &octet, 1)) == -1)
		return -1;
	*dest |= octet;
	return ret;
}

int filer_read_word32(FILER_FD fd, Uint32 *dest) {
	Uint16 word16;
	int ret;
	if (filer_read_word16(fd, &word16) == -1)
		return -1;
	*dest = word16 << 16;
	if ((ret = filer_read_word16(fd, &word16)) == -1)
		return -1;
	*dest |= word16;
	return ret;
}
