/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "pia.h"
#include "joystick.h"
#include "logging.h"
#include "types.h"

Uint32 joystick_rightx, joystick_righty;
Uint32 joystick_leftx, joystick_lefty;

void joystick_init(void) {
	joystick_rightx = joystick_righty = 128;
	PIA_1A.wrfunc[1] = joystick_write_byte;
}

void joystick_reset(void) {
}

void joystick_write_byte(Uint32 octet) {
	Uint32 xcompare, ycompare;
	octet &= 0xfc;
	if (PIA_0B.control_register & 0x08) {
		xcompare = (joystick_leftx > octet);
		ycompare = (joystick_lefty > octet);
	} else {
		xcompare = (joystick_rightx > octet);
		ycompare = (joystick_righty > octet);
	}
	if (PIA_0A.control_register & 0x08) {
		if (ycompare) {
			PIA_0A.port_input |= 0x80;
		} else {
			PIA_0A.port_input &= 0x7f;
		}
	} else {
		if (xcompare) {
			PIA_0A.port_input |= 0x80;
		} else {
			PIA_0A.port_input &= 0x7f;
		}
	}
}
