/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <SDL/SDL.h>

#include "types.h"
#include "keyboard.h"
#include "joystick.h"
#include "pia.h"
#include "snapshot.h"
#include "xroar.h"
#include "logging.h"

#define KEY_PRESSED(s) keyboard_column[keymap[s][0]] &= ~(1<<keymap[s][1]); \
	keyboard_row[keymap[s][1]] &= ~(1<<keymap[s][0])
#define KEY_RELEASED(s) keyboard_column[keymap[s][0]] |= 1<<keymap[s][1]; \
	keyboard_row[keymap[s][1]] |= 1<<keymap[s][0]

SDL_Joystick *joy = NULL;
static Uint32 control = 0;

void keyboard_init_arch(void) {
	LOG_DEBUG(1,"Number of joysticks: %d\n", SDL_NumJoysticks());
	if (SDL_NumJoysticks() > 0) {
		joy = SDL_JoystickOpen(0);
		if (joy) {
			LOG_DEBUG(2,"Opened Joystick 0\n");
			LOG_DEBUG(2,"Joystick name: %s\n", SDL_JoystickName(0));
			LOG_DEBUG(2,"Number of Axes: %d\n", SDL_JoystickNumAxes(joy));
			LOG_DEBUG(2,"Number of Buttons: %d\n", SDL_JoystickNumButtons(joy));
			LOG_DEBUG(2,"Number of Balls: %d\n", SDL_JoystickNumBalls(joy));
		}
	}
}

void keyboard_poll(void) {
	SDL_Event event;
	SDLKey sym;
	/* *** JOYSTICK SUPPORT *** */
	SDL_JoystickUpdate();
	joystick_leftx = (SDL_JoystickGetAxis(joy, 3)+32768) >> 8;
	joystick_lefty = (SDL_JoystickGetAxis(joy, 2)+32768) >> 8;
	joystick_rightx = (SDL_JoystickGetAxis(joy, 0)+32768) >> 8;
	joystick_righty = (SDL_JoystickGetAxis(joy, 1)+32768) >> 8;
	if (SDL_JoystickGetButton(joy, 1)) {
		PIA_0A.tied_low &= 0xfe;
	} else {
		PIA_0A.tied_low |= 0x01;
	}
	if (SDL_JoystickGetButton(joy, 0)) {
		PIA_0A.tied_low &= 0xfd;
	} else {
		PIA_0A.tied_low |= 0x02;
	}
	/* *** END JOYSTICK SUPPORT *** */
	while (SDL_PollEvent(&event) == 1) {
		switch(event.type) {
			case SDL_KEYDOWN:
				sym = event.key.keysym.sym;
				switch (sym) {
					case SDLK_UP: KEY_PRESSED(11); break;
					case SDLK_DOWN: KEY_PRESSED(10); break;
					case SDLK_LEFT: KEY_PRESSED(8); break;
					case SDLK_RIGHT: KEY_PRESSED(9); break;
					case SDLK_LCTRL: case SDLK_RCTRL:
							 control = 1; break;
					case SDLK_LSHIFT: case SDLK_RSHIFT:
							 KEY_PRESSED(0); break;
					default: if (sym == SDLK_c && control) exit(0);
						if (sym == SDLK_s && control) { snapshot_state = 1; break; }
						if (sym == SDLK_l && control) { snapshot_state = 2; break; }
						if (sym == SDLK_r && control) { snapshot_state = 3; break; }
#ifdef TRACE
						//if (sym == SDLK_p && control) { xroar_dumpstate(); break; }
						if (sym == SDLK_t && control) { trace ^= 1; break; }
#endif
						if (sym < 128 && keymap[sym][0] != 8) {
							KEY_PRESSED(sym);
						}
				}
				break;
			case SDL_KEYUP:
				sym = event.key.keysym.sym;
				switch (sym) {
					case SDLK_UP: KEY_RELEASED(11); break;
					case SDLK_DOWN: KEY_RELEASED(10); break;
					case SDLK_LEFT: KEY_RELEASED(8); break;
					case SDLK_RIGHT: KEY_RELEASED(9); break;
					case SDLK_LCTRL: case SDLK_RCTRL:
							 control = 0; break;
					case SDLK_LSHIFT: case SDLK_RSHIFT:
							 KEY_RELEASED(0); break;
					default:
						if (sym < 128 && keymap[sym][0] != 8) {
							KEY_RELEASED(sym);
						}
						break;
				}
				break;
			default:
				break;
		}
	}
	keyboard_column_write(keyboard_last_column_mask);
	keyboard_row_write(keyboard_last_row_mask);
}
