/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "machine.h"
#include "keyboard.h"
#include "logging.h"
#include "filer.h"
#include "types.h"

Uint32 machine_mode;
Uint32 brk_csrdon, brk_bitin;
Uint8 ram0[0x8000];
Uint8 ram1[0x8000];
Uint8 rom0[0x8000];

static char *rom_names[2][4] = {
	{ "dragon.rom", "dragon.dgn", "d32rom.dgn", "dragrom.dgn" },
	{ "coco.rom", "coco_pa.rom", "cocodisk.rom", "coco.dgn" }
};

static Uint32 dragon_keymap[128][2] = {
	{7,6}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 0 - 7 */
	{5,5}, {6,5}, {4,5}, {3,5}, {1,6}, {0,6}, {8,8}, {8,8}, /* 8 - 15 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 16 - 23 */
	{8,8}, {8,8}, {8,8}, {2,6}, {8,8}, {8,8}, {8,8}, {8,8}, /* 24 - 31 */
	{7,5}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 32 - 39 */
	{8,8}, {8,8}, {8,8}, {8,8}, {4,1}, {2,1}, {6,1}, {7,1}, /* 40 - 47 */
	{0,0}, {1,0}, {2,0}, {3,0}, {4,0}, {5,0}, {6,0}, {7,0}, /* 48 - 55 */
	{0,1}, {1,1}, {8,8}, {3,1}, {8,8}, {5,1}, {8,8}, {8,8}, /* 56 - 63 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 64 - 71 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 72 - 79 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 80 - 87 */
	{8,8}, {8,8}, {8,8}, {0,2}, {2,6}, {8,8}, {5,5}, {8,8}, /* 88 - 95 */
	{1,6}, {1,2}, {2,2}, {3,2}, {4,2}, {5,2}, {6,2}, {7,2}, /* 96 - 103 */
	{0,3}, {1,3}, {2,3}, {3,3}, {4,3}, {5,3}, {6,3}, {7,3}, /* 104 - 111 */
	{0,4}, {1,4}, {2,4}, {3,4}, {4,4}, {5,4}, {6,4}, {7,4}, /* 112 - 119 */
	{0,5}, {1,5}, {2,5}, {8,8}, {8,8}, {8,8}, {8,8}, {1,6}, /* 120 - 127 */
};
static Uint32 coco_keymap[128][2] = {
	{7,6}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 0 - 7 */
	{5,3}, {6,3}, {4,3}, {3,3}, {1,6}, {0,6}, {8,8}, {8,8}, /* 8 - 15 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 16 - 23 */
	{8,8}, {8,8}, {8,8}, {2,6}, {8,8}, {8,8}, {8,8}, {8,8}, /* 24 - 31 */
	{7,3}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 32 - 39 */
	{8,8}, {8,8}, {8,8}, {8,8}, {4,5}, {2,5}, {6,5}, {7,5}, /* 40 - 47 */
	{0,4}, {1,4}, {2,4}, {3,4}, {4,4}, {5,4}, {6,4}, {7,4}, /* 48 - 55 */
	{0,5}, {1,5}, {8,8}, {3,5}, {8,8}, {5,5}, {8,8}, {8,8}, /* 56 - 63 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 64 - 71 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 72 - 79 */
	{8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, {8,8}, /* 80 - 87 */
	{8,8}, {8,8}, {8,8}, {0,0}, {2,6}, {8,8}, {5,3}, {8,8}, /* 88 - 95 */
	{1,6}, {1,0}, {2,0}, {3,0}, {4,0}, {5,0}, {6,0}, {7,0}, /* 96 - 103 */
	{0,1}, {1,1}, {2,1}, {3,1}, {4,1}, {5,1}, {6,1}, {7,1}, /* 104 - 111 */
	{0,2}, {1,2}, {2,2}, {3,2}, {4,2}, {5,2}, {6,2}, {7,2}, /* 112 - 119 */
	{0,3}, {1,3}, {2,3}, {8,8}, {8,8}, {8,8}, {8,8}, {1,6}, /* 120 - 127 */
};
Uint32 keymap[128][2];

void machine_init(void) {
	machine_mode = MACHINE_DRAGON;  /* machine_reset will set stuff up */
	memset(keymap, 0, sizeof(keymap));
}

void machine_reset(void) {
	memset(ram0, 0, sizeof(ram0));
	memset(ram1, 0, sizeof(ram1));
	if (machine_set_mode(machine_mode) == -1)
		machine_set_mode(machine_mode^1);
}

static void machine_hardware(Uint32 mode) {
	machine_mode = mode;
	switch (mode) {
		default:
		case MACHINE_DRAGON:
			brk_csrdon = 0xbde7; brk_bitin = 0xbda5;
			memcpy(keymap, dragon_keymap, sizeof(keymap));
			break;
		case MACHINE_COCO:
			brk_csrdon = 0xa77c; brk_bitin = 0xa5c9;
			memcpy(keymap, coco_keymap, sizeof(keymap));
			break;
	}
/*
#ifdef GP32
	matrix_enter = &keyboard_matrix[keymap[13][0]][keymap[13][1]];
	matrix_shift = &keyboard_matrix[keymap[0][0]][keymap[0][1]];
	matrix_space = &keyboard_matrix[keymap[32][0]][keymap[32][1]];
	matrix_up = &keyboard_matrix[keymap[11][0]][keymap[11][1]];
	matrix_down = &keyboard_matrix[keymap[10][0]][keymap[10][1]];
	matrix_left = &keyboard_matrix[keymap[8][0]][keymap[8][1]];
	matrix_right = &keyboard_matrix[keymap[9][0]][keymap[9][1]];
#endif
*/
}

int machine_load_rom(char *filename) {
	FILER_FD fd;
	if (filer_open(filename, FILER_READ, &fd) == -1)
		return -1;
	filer_read(fd, rom0, 16);
	if (rom0[0] == 0xa1) {
		filer_read(fd, rom0, sizeof(rom0));
	} else {
		filer_read(fd, rom0+16, sizeof(rom0)-16);
	}
	filer_close(fd);
	if (rom0[0] == 0x58 && rom0[1] == 0x45)
		machine_hardware(MACHINE_COCO);
	else
		machine_hardware(MACHINE_DRAGON);
	return 0;
}

int machine_set_mode(Uint32 mode) {
	int i;
	for (i = 0; i < 4; i++) {
		if (machine_load_rom(rom_names[mode][i]) != -1)
			return 0;
	}
	return -1;
}
