/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  See LICENSE for redistribution conditions. */

#ifndef __PIA_H__
#define __PIA_H__

#include "types.h"
#include "m6809.h"

#define MAX_WRFUNCS 4

typedef void (*write_function)(Uint32 octet);
typedef struct {
	Uint32 control_register;
	Uint32 direction_register;
	Uint32 output_register;
	Uint32 port_input;
	Uint32 tied_low;
	/* Convenience flags split out from control_register above */
	Uint32 interrupt_enable;
	Uint32 interrupt_transition;
	Uint32 register_select;  /* 0 = DDR, SET = OR */
	/* ignore Cx2 stuff for now */
	write_function wrfunc[MAX_WRFUNCS];
} pia_port;

extern pia_port PIA_0A, PIA_0B, PIA_1A, PIA_1B;

#define PIA_TIE_LOW(p,l) p.tied_low &= ~l
#define PIA_UNTIE_LOW(p,l) p.tied_low |= l

#define PIA_SET_P0CA1 if (PIA_0A.interrupt_enable) { PIA_0A.control_register |= 0x80; m6809_irq(); }
#define PIA_SET_P0CB1 if (PIA_0B.interrupt_enable) { PIA_0B.control_register |= 0x80; m6809_irq(); }
#define PIA_SET_P1CA1 if (PIA_1A.interrupt_enable) { PIA_1A.control_register |= 0x80; m6809_firq(); }
#define PIA_SET_P1CB1 if (PIA_1B.interrupt_enable) { PIA_1B.control_register |= 0x80; m6809_firq(); }

void pia_init(void);
void pia_reset(void);
void pia_control_write(pia_port *p, Uint32 octet);
Uint32 pia_control_read(pia_port *p);
/* This is for MPU writes to the PIA */
void pia_data_write(pia_port *p, Uint32 octet);
Uint32 pia_data_read(pia_port *p);
/* This is called by devices to set input bits */
void pia_data_set(pia_port *p, Uint32 octet);

#endif  /* __PIA_H__ */
