/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "video.h"
#include "types.h"

static PIXEL backup[320*240];

GPDRAWSURFACE screen[2];
#ifdef DOUBLE_BUFFERED
Uint32 flip;
#endif

void video_init(void) {
	GpGraphicModeSet(VIDEO_BPP, NULL);
	GpLcdSurfaceGet(&screen[0], 0);
#ifdef DOUBLE_BUFFERED
	GpLcdSurfaceGet(&screen[1], 0);
	flip = 1;
#endif
	GpSurfaceSet(&screen[0]);
	GpLcdEnable();
	video_clear_screen(MAPCOLOUR(0,0,0));
	video_fillrect(0,0,320,46,MAPCOLOUR(255,255,255));
}

void video_fillrect(Uint32 x, Uint32 y, Uint32 w, Uint32 h, PIXEL colour) {
	PIXEL *p1 = (PIXEL *)screen[0].ptbuffer + x * 240 + y;
#ifdef DOUBLE_BUFFERED
	PIXEL *p2 = (PIXEL *)screen[1].ptbuffer + x * 240 + y;
#endif
	Uint32 skip = 240 - h;
	Uint32 j;
	for (; w; w--) {
		for (j = h; j; j--) {
			*(p1++) = colour;
#ifdef DOUBLE_BUFFERED
			*(p2++) = colour;
#endif
		}
		p1 += skip;
#ifdef DOUBLE_BUFFERED
		p2 += skip;
#endif
	}
}

void video_blit_bothbufs(Uint32 dest, PIXEL *src, Uint32 width, Uint32 height) {
	PIXEL *p1 = (PIXEL *)screen[0].ptbuffer + dest;
#ifdef DOUBLE_BUFFERED
	PIXEL *p2 = (PIXEL *)screen[1].ptbuffer + dest;
#endif
	Uint32 y;
	for (; width; width--) {
		for (y = height; y; y--) {
#ifdef DOUBLE_BUFFERED
			*(p2++) = *src;
#endif
			*(p1++) = *(src++);
		}
		p1 += (240-height);
#ifdef DOUBLE_BUFFERED
		p2 += (240-height);
#endif
	}
}

void video_blit_static(int x, int y, PIXEL *src, int w, int h) {
	PIXEL *p1 = VIDEO_SCREENBASE + (x * 240) + y;
#ifdef DOUBLE_BUFFERED
	PIXEL *p2 = (PIXEL *)screen[flip^1].ptbuffer + (x * 240) + y;
#endif
	int j;
	for (; w; w--) {
		for (j = h; j; j--) {
#ifdef DOUBLE_BUFFERED
			*(p2++) = *src;
#endif
			*(p1++) = *(src++);
		}
		p1 += (240 - h);
#ifdef DOUBLE_BUFFERED
		p2 += (240 - h);
#endif
	}
}

void video_blit_bytes(Uint32 dest, PIXEL *src, Uint32 width, Uint32 height) {
	PIXEL *p = VIDEO_SCREENBASE + dest;
	Uint32 y;
	for (; width; width--) {
		for (y = height; y; y--) {
			*(p++) = *(src++);
		}
		p += (240-height);
	}
}

void video_clear_screen(PIXEL colour) {
        video_fillrect(0, 0, 320, 240, colour);
}

void video_backup(PIXEL *base) {
	memcpy(backup, base, 320*240*sizeof(PIXEL));
}

void video_restore(PIXEL *base) {
	memcpy(base, backup, 320*240*sizeof(PIXEL));
}
