/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xroar.h"
#include "machine.h"
#include "joystick.h"
#include "keyboard.h"
#include "m6809.h"
#include "pia.h"
#include "wd2797.h"
#include "sam.h"
#include "sound.h"
#include "vdg.h"
#include "filer.h"
#include "snapshot.h"
#include "tape.h"
#include "logging.h"

#ifdef TRACE
int trace = 0;
#endif

void xroar_init(void) {
	/* Initialise everything */
	machine_init();
	filer_init();
	sam_init();
	/* pia_init() must be called before devices that connect to PIAs */
	pia_init();
	wd2797_init();
	m6809_init();
	vdg_init();
	keyboard_init();
	joystick_init();
	sound_init();
	tape_init();
}

void xroar_reset(void) {
	/* Reset everything */
	machine_reset();
	keyboard_reset();
	joystick_reset();
	pia_reset();
	wd2797_reset();
	sam_reset();
	m6809_reset();
	vdg_reset();
	sound_reset();
	tape_reset();
}

void xroar_mainloop(void) {
	while (1) {
		m6809_cycle();
		switch(snapshot_state) {
			case 1: write_snapshot(); break;
			case 2: read_snapshot(); break;
			case 3: xroar_reset(); break;
		}
		snapshot_state = 0;
	}
}
